%================================
% Kalman filter example
%================================
T0 = 0.1; % sampling interval
sigVel = 2; % standard deviation doppler sensor
sigAcc = 0.01; % standard deviation accelerometer
delAcc = 0.01; % offset error
Q = [0 0;0 0]; % system error(total confidence in the model)
R = [sigVel^2 0;0 sigAcc^2]; %measurement error variances
Phi = [1 T0;0 1]; % transition matrix
H = [1 0;0 1]; % output matrix


%===================================
% Simulation Initialization
%===================================
N = 1000; % # of simulation steps
rand('state',0);
x = zeros(2,N); % initialize state vectors
zv = randn(1,N);% velocity measurement simulation
zv = (zv-mean(zv))/std(zv); zv =sigVel*zv;
za = randn(1,N);% acceleration measurement simulation
za = (za-mean(za))/std(za); za = sigAcc*za + delAcc;
z = [zv;za];

%======================================
% Kalman filter algorithm
%======================================
x(:,1)=[0;0]; % initial state 
Pprev = [4 0;0 0.0001]; % initial covariance

%======================================
% Recursive procedure
%======================================
for k=2:N
    x(:,k) = Phi*x(:,k-1); % state extrapolation
    Pk = Phi*Pprev*Phi'+Q; %covariance extrapolation
    K = Pk*H'*inv(H*Pk*H'+R); %kalman gain
    x(:,k) = x(:,k)+K*(z(:,k)-H*x(:,k)); % update state
    Pprev = (eye(2)-K*H)*Pk;% update covariance
end

%======================================
% Results plotting
%======================================

subplot(4,1,1)
t = (0:N-1)*T0;
plot(t,zv)
grid on
ylabel('z_v (m/s)')
subplot(4,1,2)
plot(t,za)
grid on
ylabel('z_a (m/s^2)')
subplot(4,1,3)
plot(t,x(1,:))
grid on
ylabel('v_{kalman} (m/s)')
subplot(4,1,4)
plot(t,x(2,:))
grid on
ylabel('a_{kalman} (m/s^2)')